@@ -318,6 +318,9 @@ class Module(EventListener):
         instance = injectable.get_instance()
         return cast(cls, instance)
 
+    def get_lazy_instance(self, cls: type[_T]) -> Lazy[_T | None]:
+        return Lazy(lambda: self.get_instance(cls))
+
     def update(self, classes: Iterable[type], injectable: Injectable):
         self.__container.update(classes, injectable)
         return self
@@ -547,12 +550,8 @@ class InjectableDecorator:
         /,
         *,
         on: type | Iterable[type] = None,
-        auto_inject: bool = True,
     ):
         def decorator(wp):
-            if auto_inject:
-                wp = self.__module.inject(wp)
-
             @lambda fn: fn()
             def classes():
                 if cls := self.__get_target_class(wp):
@@ -560,12 +559,17 @@ class InjectableDecorator:
 
                 if on is None:
                     return
-                elif isinstance(on, Iterable):
-                    yield from on
-                else:
+                elif isinstance(on, type | str):
                     yield on
+                else:
+                    yield from on
 
-            injectable = self.__injectable_type(wp)
+            @self.__module.inject
+            @wraps(wp, updated=())
+            def factory(*args, **kwargs):
+                return wp(*args, **kwargs)
+
+            injectable = self.__injectable_type(factory)
             self.__module.update(classes, injectable)
 
             return wp
